@@ -44,10 +44,9 @@ typedef struct {
   unsigned int connect_timeout;
   int active;
   int automatic_close;
-  int connected;
   int initialized;
   int refcount;
-  int freed;
+  int closed;
   MYSQL *client;
 } mysql_client_wrapper;
 
