@@ -3,7 +3,7 @@ require './spec/spec_helper.rb'
 
 RSpec.describe Mysql2::Statement do
   before :each do
-    @client = Mysql2::Client.new(DatabaseCredentials['root'].merge(:encoding => "utf8"))
+    @client = new_client(:encoding => "utf8")
   end
 
   def stmt_count
@@ -61,6 +61,12 @@ RSpec.describe Mysql2::Statement do
     expect(rows).to eq([{ "1" => 1 }])
   end
 
+  it "should handle booleans" do
+    stmt = @client.prepare('SELECT ? AS `true`, ? AS `false`')
+    result = stmt.execute(true, false)
+    expect(result.to_a).to eq(['true' => 1, 'false' => 0])
+  end
+
   it "should handle bignum but in int64_t" do
     stmt = @client.prepare('SELECT ? AS max, ? AS min')
     int64_max = (1 << 63) - 1
@@ -144,7 +150,8 @@ RSpec.describe Mysql2::Statement do
     if RUBY_VERSION =~ /1.8/
       expect(result.first['a'].strftime('%F %T %z')).to eql(now.strftime('%F %T %z'))
     else
-      expect(result.first['a'].strftime('%F %T.%6N %z')).to eql(now.strftime('%F %T.%6N %z'))
+      # microseconds is six digits after the decimal, but only test on 5 significant figures
+      expect(result.first['a'].strftime('%F %T.%5N %z')).to eql(now.strftime('%F %T.%5N %z'))
     end
   end
 
@@ -155,7 +162,8 @@ RSpec.describe Mysql2::Statement do
     if RUBY_VERSION =~ /1.8/
       expect(result.first['a'].strftime('%F %T %z')).to eql(now.strftime('%F %T %z'))
     else
-      expect(result.first['a'].strftime('%F %T.%6N %z')).to eql(now.strftime('%F %T.%6N %z'))
+      # microseconds is six digits after the decimal, but only test on 5 significant figures
+      expect(result.first['a'].strftime('%F %T.%5N %z')).to eql(now.strftime('%F %T.%5N %z'))
     end
   end
 
@@ -326,18 +334,19 @@ RSpec.describe Mysql2::Statement do
   end
 
   context "#fields" do
-    before(:each) do
-      @client.query "USE test"
-      @test_result = @client.prepare("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").execute
-    end
-
     it "method should exist" do
-      expect(@test_result).to respond_to(:fields)
+      stmt = @client.prepare("SELECT 1")
+      expect(stmt).to respond_to(:fields)
     end
 
     it "should return an array of field names in proper order" do
-      result = @client.prepare("SELECT 'a', 'b', 'c'").execute
-      expect(result.fields).to eql(%w(a b c))
+      stmt = @client.prepare("SELECT 'a', 'b', 'c'")
+      expect(stmt.fields).to eql(%w(a b c))
+    end
+
+    it "should return nil for statement with no result fields" do
+      stmt = @client.prepare("INSERT INTO mysql2_test () VALUES ()")
+      expect(stmt.fields).to eql(nil)
     end
   end
 
@@ -367,36 +376,47 @@ RSpec.describe Mysql2::Statement do
       expect(@test_result['tiny_int_test']).to eql(1)
     end
 
-    it "should return TrueClass or FalseClass for a TINYINT value if :cast_booleans is enabled" do
-      @client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES (1)'
-      id1 = @client.last_id
-      @client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES (0)'
-      id2 = @client.last_id
-      @client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES (-1)'
-      id3 = @client.last_id
+    context "cast booleans for TINYINT if :cast_booleans is enabled" do
+      # rubocop:disable Style/Semicolon
+      let(:client) { new_client(:cast_booleans => true) }
+      let(:id1) { client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES ( 1)'; client.last_id }
+      let(:id2) { client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES ( 0)'; client.last_id }
+      let(:id3) { client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES (-1)'; client.last_id }
+      # rubocop:enable Style/Semicolon
 
-      result1 = @client.query 'SELECT bool_cast_test FROM mysql2_test WHERE bool_cast_test = 1 LIMIT 1', :cast_booleans => true
-      result2 = @client.query 'SELECT bool_cast_test FROM mysql2_test WHERE bool_cast_test = 0 LIMIT 1', :cast_booleans => true
-      result3 = @client.query 'SELECT bool_cast_test FROM mysql2_test WHERE bool_cast_test = -1 LIMIT 1', :cast_booleans => true
-      expect(result1.first['bool_cast_test']).to be true
-      expect(result2.first['bool_cast_test']).to be false
-      expect(result3.first['bool_cast_test']).to be true
+      after do
+        client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2},#{id3})"
+      end
 
-      @client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2},#{id3})"
+      it "should return TrueClass or FalseClass for a TINYINT value if :cast_booleans is enabled" do
+        query = client.prepare 'SELECT bool_cast_test FROM mysql2_test WHERE id = ?'
+        result1 = query.execute id1
+        result2 = query.execute id2
+        result3 = query.execute id3
+        expect(result1.first['bool_cast_test']).to be true
+        expect(result2.first['bool_cast_test']).to be false
+        expect(result3.first['bool_cast_test']).to be true
+      end
     end
 
-    it "should return TrueClass or FalseClass for a BIT(1) value if :cast_booleans is enabled" do
-      @client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (1)'
-      id1 = @client.last_id
-      @client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (0)'
-      id2 = @client.last_id
+    context "cast booleans for BIT(1) if :cast_booleans is enabled" do
+      # rubocop:disable Style/Semicolon
+      let(:client) { new_client(:cast_booleans => true) }
+      let(:id1) { client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (1)'; client.last_id }
+      let(:id2) { client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (0)'; client.last_id }
+      # rubocop:enable Style/Semicolon
 
-      result1 = @client.query "SELECT single_bit_test FROM mysql2_test WHERE id = #{id1}", :cast_booleans => true
-      result2 = @client.query "SELECT single_bit_test FROM mysql2_test WHERE id = #{id2}", :cast_booleans => true
-      expect(result1.first['single_bit_test']).to be true
-      expect(result2.first['single_bit_test']).to be false
+      after do
+        client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2})"
+      end
 
-      @client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2})"
+      it "should return TrueClass or FalseClass for a BIT(1) value if :cast_booleans is enabled" do
+        query = client.prepare 'SELECT single_bit_test FROM mysql2_test WHERE id = ?'
+        result1 = query.execute id1
+        result2 = query.execute id2
+        expect(result1.first['single_bit_test']).to be true
+        expect(result2.first['single_bit_test']).to be false
+      end
     end
 
     it "should return Fixnum for a SMALLINT value" do
@@ -453,39 +473,39 @@ RSpec.describe Mysql2::Statement do
 
       it "should return DateTime when timestamp is < 1901-12-13 20:45:52" do
         # 1901-12-13T20:45:52 is the min for 32bit Ruby 1.8
-        r = @client.query("SELECT CAST('1901-12-13 20:45:51' AS DATETIME) as test")
+        r = @client.prepare("SELECT CAST('1901-12-13 20:45:51' AS DATETIME) as test").execute
         expect(r.first['test']).to be_an_instance_of(klass)
       end
 
       it "should return DateTime when timestamp is > 2038-01-19T03:14:07" do
         # 2038-01-19T03:14:07 is the max for 32bit Ruby 1.8
-        r = @client.query("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test")
+        r = @client.prepare("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test").execute
         expect(r.first['test']).to be_an_instance_of(klass)
       end
     elsif 1.size == 8 # 64bit
       if RUBY_VERSION =~ /1.8/
         it "should return Time when timestamp is > 0138-12-31 11:59:59" do
-          r = @client.query("SELECT CAST('0139-1-1 00:00:00' AS DATETIME) as test")
+          r = @client.prepare("SELECT CAST('0139-1-1 00:00:00' AS DATETIME) as test").execute
           expect(r.first['test']).to be_an_instance_of(Time)
         end
 
         it "should return DateTime when timestamp is < 0139-1-1T00:00:00" do
-          r = @client.query("SELECT CAST('0138-12-31 11:59:59' AS DATETIME) as test")
+          r = @client.prepare("SELECT CAST('0138-12-31 11:59:59' AS DATETIME) as test").execute
           expect(r.first['test']).to be_an_instance_of(DateTime)
         end
 
         it "should return Time when timestamp is > 2038-01-19T03:14:07" do
-          r = @client.query("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test")
+          r = @client.prepare("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test").execute
           expect(r.first['test']).to be_an_instance_of(Time)
         end
       else
         it "should return Time when timestamp is < 1901-12-13 20:45:52" do
-          r = @client.query("SELECT CAST('1901-12-13 20:45:51' AS DATETIME) as test")
+          r = @client.prepare("SELECT CAST('1901-12-13 20:45:51' AS DATETIME) as test").execute
           expect(r.first['test']).to be_an_instance_of(Time)
         end
 
         it "should return Time when timestamp is > 2038-01-19T03:14:07" do
-          r = @client.query("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test")
+          r = @client.prepare("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test").execute
           expect(r.first['test']).to be_an_instance_of(Time)
         end
       end
@@ -524,10 +544,9 @@ RSpec.describe Mysql2::Statement do
           result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['enum_test'].encoding).to eql(Encoding::UTF_8)
 
-          client2 = Mysql2::Client.new(DatabaseCredentials['root'].merge(:encoding => 'ascii'))
+          client2 = new_client(:encoding => 'ascii')
           result = client2.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['enum_test'].encoding).to eql(Encoding::US_ASCII)
-          client2.close
         end
       end
 
@@ -557,10 +576,9 @@ RSpec.describe Mysql2::Statement do
           result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['set_test'].encoding).to eql(Encoding::UTF_8)
 
-          client2 = Mysql2::Client.new(DatabaseCredentials['root'].merge(:encoding => 'ascii'))
+          client2 = new_client(:encoding => 'ascii')
           result = client2.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['set_test'].encoding).to eql(Encoding::US_ASCII)
-          client2.close
         end
       end
 
@@ -651,10 +669,9 @@ RSpec.describe Mysql2::Statement do
               result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
               expect(result[field].encoding).to eql(Encoding::UTF_8)
 
-              client2 = Mysql2::Client.new(DatabaseCredentials['root'].merge(:encoding => 'ascii'))
+              client2 = new_client(:encoding => 'ascii')
               result = client2.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
               expect(result[field].encoding).to eql(Encoding::US_ASCII)
-              client2.close
             end
           end
 
